#################################
# set parameters needed for STILT
#################################
#---------------------------------------------------------------------------------------------------
# $Id: setStiltparam.r,v 1.8 2007/10/12 19:01:46 trn Exp $
#---------------------------------------------------------------------------------------------------

###### set directories ######
###### make sure they exist, at least the ones in the following first paragraph
path <- "./"                                            # path where output gets saved,
                                                        # also input data (Receptor locations and times)
                                                        # and boundary mixing ratio objects are read from 'path'
metpath <- "/group/wofsy/stilt/Metdata/"                # where met data are stored in ARL format
vegpath <- "/group/wofsy/stilt/Vegetation/"             # path to get to surface fluxes and vegetation grids at various resolutions
rundir <- "./Exe/"                                      # specifies main directory where different directories are found to run hymodelc
shlibpath <- "/Net/Groups/BSY/STILT/repository_export/stiltR/shlib_x64_linux"   # R extensions

### optional directories
evilswipath="/project/p1229/EVILSWI2004FULL/"           # VPRM only -- input path for EVI and LSWI
vprmconstantspath="/project/p1229/VPRMConstants/"       # VPRM only -- input path for file with VPRM constants
nldaspath="/project/p1229/Radiation/NLDAS/"             # VPRM only -- input path for NLDAS temperature and radiation

###### basic parameters ######
nhrs <- -24             # for how many hours to run (time-backward runs negative).
nparstilt <- 100        # how many particles per receptor? 100 of for receptor oriented modeling with dynamic grid resolution is ok
veght <- 0.5            # surface layer (for fluxes); if less than 1 then as fraction of mixed layer height; 0.5 is a good value.
convect <- F            # T for convection (RAMS winds: grell convection scheme, EDAS and FNL: simple redistribution within vertical range with CAPE>0)
stepsize <- 0           # Enforces Courant criterion. When >0: maximum horizontal distances travelled by a particle in a single timestep, in km.
                        # For dynamic resolution, choose value 0. First 12 hrs: 20 km; then 60 km

                        # transport error due to wind error or mixing height error included as stochastic prozess?
                        # then need parameters for covariance matrices; otherwise set parameters to NULL
siguverr=NULL           # stddev of magnitude in horizontal wind errors [m/s]
TLuverr=NULL            # correlation timescale [min]
horcoruverr=NULL        # horizontal correlation lengthscale of horizontal winds [km]
zcoruverr=NULL          # vertical correlation lengthscale of horizontal winds [km]
sigzierr=NULL           # stddev of magnitude in mixed layer height errors [%]
TLzierr=NULL            # correlation timescale [min]
horcorzierr=NULL        # horizontal correlation lengthscale of mixed layer height errors [km]
ziscale=NULL            # scaling factor(s) for PBL height; NULL for default; -1 to use model pbl heights

metsource <- c("edas", "fnl")   # Source of Meteorological data, for analysis runs (not forecasts)
         #metsource      generating model                      dt    dx [km]      file duration  filename example
         #"edas"         Eta Data Assimilation System, NCEP    3h    80           0.5m           edas.subgrd.apr00.001
         #"edas40"       NAM (Eta) Data Assimilation System    3h    40           0.5m           edas.apr04.002
         #"fnl"          Global Data Assim, Syst. (GDAS) NCEP  6h    180          0.5m           fnl.nh.apr00.001           
         #"fnl.nh"        - same (default is nh)               6h    180          0.5m           fnl.nh.apr00.001
         #"fnl.sh"        - southern hem.                      6h    180          0.5m           fnl.sh.apr00.001
         #"brams"        Brazilian implementation of RAMS      ~0.3h 40           1d             brams_12_15_2003_1.bin
         #"ecmw"         ECMWF                                 3h    35 (->2/06)  1d             ecmw.050511.arl
         #                                                           25 (2/06->)  1m        
         #"ECmetF"       ECMWF, patched short term forecasts   3h    35 (->2/06)  72 - 144 h     ECmetF.05070100.arl
         #                                                           25 (2/06->)  
         #"alad"         Aladin meso. forecasts (MeteoFrance)  3h    8            72h            aladinF.07042100.arl
         #"wrf"          Weather Research & Forecasting        0.3h  2-50         ?h             d01.20051223.arl
         #"d01", "do2",          WRF nested domains            0.3h  2-50         ?h             d01.20051223.arl   



###### define receptors ######
Timesname <- "Times.hf" # name of object containing frac. julian, lat, lon, agl (km) as receptor information;
                        # this object needs to be saved (use assignr) in directory 'path' (see above)
                        # Times.hf was created with create_times.r
totpart=10              # in how many parts to split the whole object (usefull for distribution to different processors)
part=1                  # which part (of totpart) to run
if ("num.txt"%in%dir()) {part <- as.numeric(scan("num.txt")); unix("rm num.txt")} 
                        # get part from num.txt, which is writen by stiltnum.bat, remove after reading
ist=0                   # for short test run; "i-start"= row of Timesname object to start with; set to 0 for full run
ien=3; if (ist == 0) ien <- 0   # for short test run; "i-end"= row of Timesname object to end with

#####################################
####### PARTICLE LOCATIONS ##########
###### control parameters for what STILT should do: Particle locations, mixing ratios or footprints ######
varstrajec <- c("time", "index", "lat", "lon", "agl", "grdht", "foot", "temp0", "swrad", "zi",
                "dens", "pres", "dmass")
      # specifies output variables from Trajec(); these will be output along the trajectory.
      # can be any subset of:
      # c("time", "sigmaw", "TL", "lon", "lat", "agl", "grdht", "index", "cldidx", "temp0",
      #  "sampt", "foot", "shtf", "lhtf", "tcld", "dmass", "dens", "rhf", "solw", "lcld",
      #  "zloc", "swrad", "wbar", "zi", "totrain", "convrain", "zconv")
overwrite=T             # T: rerun hymodelc, even if particle location object for same starting location and time is found;
                        # F: re-use previously calculated particle location object
remove.Trajfile=F       # T: remove trajectory files after VPRM (to prevent accumulation of large files) (default: F)
create.X0=F             # T: Save zero-time values in separate initial trajectory file (default: F)


################################
####### MIXING RATIOS ##########
fluxTF <- T                     # mapping to emission fluxes and "stupid biosphere" CO2 fluxes
fluxtracers <- c("co", "co2")   # vector of names for which mixing ratios are calculated; any subset of c("co", "co2", "ch4", "h2")
aggregation <- 1                # degrade resolution (for aggregation error): 0: use only highest resolution at 20 km;
        #        1-16: dynamic resolution.
        #  1: starts with highest resolution of 20 km, then degrades resolution as footprint size increases
        #  2-16: starts with coarser resolution (see table below)
        # aggregation:  1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16
        # coarser in x: 1 1 2 2 2 4 4 4 8  8  8 16 16 16 32 32 # factors by which grids have been made coarser
        # coarser in y: 1 2 1 2 4 2 4 8 4  8 16  8 16 32 16 32 # e.g., '4' means grid is 4 times coarser
if (stepsize == 0 & aggregation == 0) {aggregation <- 1; print("override: aggregation changed to dynamic for stepsize=0!")}
        
#*******Choice of biosphere -- either GSB (Greatly simplified biosphere) or VPRM (Vegetation photosynthesis and respiration model
# The GSB is appropriate where a simple surface flux model is needed; VPRM is appropriate where most accurate surface fluxes are desired
# Note: The GSB is fairly computationally efficient; the VPRM is not.
fluxmod <- "GSB"                # Which surface flux model to use: "GSB" or "VPRM"
detailsTF <- FALSE              # detailsTF' if TRUE, for each particle the flux contribution is saved in a big object
                                # WARNING: setting detailsTF=TRUE may result in large files being created and stored

# GSB options
linveg <- FALSE                 # if TRUE, CO2 fluxes are represented as linear functions of temperature and radiation (otherwise GEE is non linear)

# VPRM options
usenldasrad <- FALSE            # If TRUE, nldas radiation is read from a library and used to drive the model; If FALSE output from assimilated is used
usenldastemp <- FALSE           # If TRUE, nldas temperature is read from a library and used to drive the model; If FALSE output from assimilated is used
pre2004 <- FALSE                # For conversion of shortwave radiation to PAR; the assimilated models (specifically edas) changed at the beginnning of 2004
        # this specifies which conversion factor to use.
keepevimaps <- FALSE            # this assigns evi and lswi maps to the global environment -- saving significant I/O processing time
        #**BE CAREFUL** when using this, as it will put a number of LARGE objects in your database. If dynamic memory
        # allocation problems are anticipated, this feature should NOT be used!!!


###########################################
####### FOOTPRINTS AND INFLUENCE ##########
# Output will be a 3 D array of Influence (or Surface Influence/Footprints)
# these objects are given names that reflect starting location and time
# e.g. "foot2002x08x01x00x42.54Nx072.17Wx00030"
# NOTE: Storage requirement for one footprint timestep is about 1 Mb, 
# so for time resolved footprints over ~weeks this can easily reach 
# 200 Mb per receptor
footprintTF <- F        # calculate footprint (surface influence) or "volume" influence
foottimes <- c(0,10,20) # vector of times (backtimes) in hours between which footprint is computed
zbot <- 0               # lower vertical bound for influence projection, in meters agl
ztop <- 0               # upper vertical bound for influence projection, in meters agl
                        # if ztop set to zero, surface influence will be calculated
footplotTF <- F         # plotting of footprints; will be plotted to png file in directory path + "/Footprints"
numpix.x <- 376         # number of pixels in x directions in grid
numpix.y <- 324         # number of pixels in y directions in grid
lon.ll <- -145          # lower left corner of grid
lat.ll <- 11            # lower left corner of grid
lon.res <- 1/4          # resolution in degrees longitude
lat.res <- 1/6          # resolution in degrees latitude
landcov <- "IGBP"       # which landcover to use?
        # IGBP -- Original IGBP, from Christoph
        # GLCC -- Original VPRM, based on updated IGBP (GLC2.0)
        # DVN -- The updated Devan system
        #       'SYNMAP' Martin Jung synmap product
        #       'SYNMAP.VPRM8' 8 VPRM classes based on synmap product
        # Note: Current (March 2007) working combinations are GSB-IGBP and VPRM-DVN; other combinations fail in 
        #      Trajecvprm or Trajecflux with an error message.

###########################################
### CO CONTRIBUTION FROM BIOMASS BURNING ##
# Set the following parameters if you are modelling CO and would like to include influence from forest fires.
# The output will be saved in its own file in the path directory.  At present, model only works for CO in 2004.
biomassburnTF <- F                              # Should the biomassburning model for CO be run?

burnpath <- "/home/smiller/biomass_burning/"    # This is the path to the biomass burning files (These files should
             # be modified as in /homes/smiller/biomassburning/modified_fireemis_script.txt).

############ set up boundary fields and OH #######################
# required only 1st time to set up grids
######## OH fields after GEOSchem (climatological values, 12 months, 2 different latitudes, parameterized pressure dependence ########
if (fluxTF) oh <- read.asc(paste(sourcepath, "OH.asc", sep=""))

# only when necessary (new tracer or new time span)
# get boundary conditions
# CO2 boundary conditions: default is "climatological"
co2inikind <- "climat"                  # possible values: "climat" (Gerbig et al. (2003)), "CT" (CarbonTracker)
                                        # "CT", then co2inifile must be set
co2inifile <- NULL


if (fluxTF & !existsr(paste(tolower(fluxtracers[1]), ".ini", sep=""), path)) {
  print(paste("setStiltparam.r: no ", fluxtracers[1], " boundary fields found!", sep=""))
             print("need to use read.bg()")
#      print("only valid for CO and CO2 from 1/1/99 to 12/31/02!!")
  read.bg(spec=c("CO2", "CO"), datename="1_1_03_12_31_04", pathin="/group/wofsy/stilt/Boundary/", pathout=path)
}

# get CO2 flux parameters
if (fluxTF) {library(foreign); data.restore(paste(sourcepath, "dlambda.veg.dmp", sep=""), print=T)}
if (fluxTF & linveg) {dlambda.simp.veg <- getr("dlambda.simp.veg", path="/Net/Groups/BSY/BSY_3/cgerbig/Rsource/polarR/")}
# object dlambda.veg contains parameters of simple light and temperature fit to Fluxnet data
# NEE=drdt * T  + a3 * swrad / (a4 + swrad)
# dlambdaGEE and dlambdaR are relative uncertainties in 3 day aggregated gross fluxes (GEE and R)
#     vgroup        a3        a4       drdt dlambdaGEE dlambdaR
#[1, ]      1 -25.19497  566.9109 0.18198077  1.4316519 2.260087
#[2, ]      2  -1.17163   95.7913 0.05482351  3.0875475 7.879271
#[3, ]      3 -64.67396 1466.6771 0.18856880  0.4396042 1.188944

######## check on surface flux grids and vegetation classification after IGBP ########
if (fluxTF) {
  if (landcov == "IGBP") {
    veghead <- "veg."
  } else if (landcov == "GLCC") {
    veghead <- "glcc."
  } else if (landcov == "DVN") {
    veghead <- "devanveg."
  } else if (landcov == "SYNMAP") {
    veghead <- "synmap."
  } else if (landcov == "SYNMAP.VPRM8") {
    veghead <- "synvprm8."
  } else {
    stop("setStiltparam: Improperly specified Landcover format; Exiting now!")
    veghead <- "veg."
  }

  if (fluxTF & !existsr(paste(veghead, "1.001", sep=""), vegpath)) {
  stop("setStiltparam: NO surface flux grids found")
  # source(paste(sourcepath, "gen_veg_mat.r", sep=""))
  }
} # if fluxTF

# Read in the required parameters for the CO biomassburning script.
if (biomassburnTF) {  
  biomassburning <- read.table(file=paste(burnpath, "modified_FireEmis_2004.txt", sep=""), header=T, row.names=NULL)
  source(paste(sourcepath, "trajecbiomassburn2.r", sep="")) }
