!$$$  SUBPROGRAM DOCUMENTATION BLOCK
!
! SUBPROGRAM:  GBLSET           GloBaL grid SETup
!   PRGMMR:    ROLAND DRAXLER   ORG: R/ARL       DATE:01-09-03
!
! ABSTRACT:  THIS CODE WRITTEN AT THE AIR RESOURCES LABORATORY ...
!   SETUPS THE PARAMETERS FOR A GLOBAL LATITUDE LONGITUDE GRID
!   THAT MAY BE USED FOR COMPUTATIONS INSTEAD OF THE CONFORMAL
!   PROJECTION SYSTEM.
!
! PROGRAM HISTORY LOG:
!   LAST REVISED: 12 Mar 2001 (RRD) - initial version
!
! USAGE:  CALL GBLSET(KG)
!
!   INPUT ARGUMENT LIST:     see below
!   OUTPUT ARGUMENT LIST:    see below
!   INPUT FILES:             none
!   OUTPUT FILES:            none
!
! ATTRIBUTES:
!   LANGUAGE: FORTRAN 90
!   MACHINE:  IBM RS6000
!
! $Id: gblset.f90,v 1.2 2007/01/31 20:49:28 trn Exp $
!
!$$$

!JCL:(07/12/2004) started out with HYSPLIT Ver. 45 Code

SUBROUTINE GBLSET(KG)

  use module_defgrid
  ! IMPLICIT NONE  !JCL: (07/09/2004) compatibility with old code
  IMPLICIT REAL*8 (A-H,O-Z)

!  INCLUDE 'DEFGRID.INC'        ! meteorology grid and file

  INTEGER, INTENT(IN) :: kg    ! active grid number

  REAL*8  :: DSX,DSY,CLAT1,CLON1,CLAT2,CLON2,DLAT,DLON

!  COMMON /GBLGRD/ GRID, DREC, FILE

  GRID(KG)%LATLON=.FALSE.
  GRID(KG)%GLOBAL=.FALSE.

! determine if this is a lat-lon grid
  IF(GRID(KG)%SIZE.EQ.0.0)THEN
     GRID(KG)%LATLON=.TRUE.
     CALL GBLDXY(KG,1.0d0,1.0d0,DSX,DSY)
     GRID(KG)%SIZE=DSY
  ELSE
     RETURN
  END IF

! find the corner points
  CLAT1=GRID(KG)%SYNC_LAT
  CLON1=GRID(KG)%SYNC_LON
  CLAT2=GRID(KG)%POLE_LAT
  CLON2=GRID(KG)%POLE_LON

! grid spacing
  DLAT=GRID(KG)%REF_LAT
  DLON=GRID(KG)%REF_LON

! determine if the grid is global
  IF(CLON2+DLON-CLON1.GE.360.0.AND.CLAT2-CLAT1.GE.180.0) GRID(KG)%GLOBAL=.TRUE.

END SUBROUTINE gblset
